@@ -27,6 +27,20 @@ class MyBool(object):
 MYTRUE = MyBool(True)
 MYFALSE = MyBool(False)
 
+import os
+### add your favorite here
+BAD_LIBC_WORKAROUND_DEFAULT = MYFALSE
+if os.name == 'posix':
+    if os.uname()[0] in ['Darwin', 'OpenBSD']:
+        BAD_LIBC_WORKAROUND_DEFAULT = MYTRUE
+
+MIN_INCOMPLETE = 100
+if os.name == 'nt':
+    from BitTorrent.platform import win_version_num
+    # starting in XP SP2 the incomplete outgoing connection limit was set to 10
+    if win_version_num >= (2, 5, 1, 2, 0):
+        MIN_INCOMPLETE = 10
+    
 from BitTorrent import languages
 
 basic_options = [
@@ -61,8 +75,10 @@ common_options = [
      _("minutes to wait between requesting more peers")),
     ('min_peers', 20,
      _("minimum number of peers to not do rerequesting")),
-    ('max_initiate', 40,
+    ('max_initiate', 60,
      _("number of peers at which to stop initiating new connections")),
+    ('max_incomplete', MIN_INCOMPLETE,
+     _("max number of outgoing incomplete connections")),
     ('max_allow_in', 80,
      _("maximum number of connections to allow, after this new incoming "
        "connections will be immediately closed")),
@@ -76,7 +92,9 @@ common_options = [
      _("the maximum number of files in a multifile torrent to keep open at a "
        "time, 0 means no limit. Used to avoid running out of file descriptors.")),
     ('start_trackerless_client', MYTRUE,
-     _("Initialize a trackerless client.  This must be enabled in order to download trackerless torrents."))
+     _("Initialize a trackerless client.  This must be enabled in order to download trackerless torrents.")),
+    ('upnp', MYTRUE,
+     _("Enable automatic port mapping")+' (UPnP)'),
     ]
 
 
@@ -116,7 +134,7 @@ rare_options = [
      _("do not connect to several peers that have the same IP address")),
     ('peer_socket_tos', 8,
      _("if nonzero, set the TOS option for peer connections to this value")),
-    ('bad_libc_workaround', MYTRUE,
+    ('bad_libc_workaround', BAD_LIBC_WORKAROUND_DEFAULT,
      _("enable workaround for a bug in BSD libc that makes file reads very slow.")),
     ('tracker_proxy', '',
      _("address of HTTP proxy to use for tracker connections")),
@@ -173,6 +191,25 @@ def get_defaults(ui):
              'local directory to look in for .torrent files to open'),
             ('ask_for_save', MYFALSE,
              'whether or not to ask for a location to save downloaded files in'),
+            ('start_minimized', MYFALSE,
+             _("Start BitTorrent minimized")),
+            ('new_version', '',
+             _("override the version provided by the http version check "
+               "and enable version check debugging mode")),
+            ('current_version', '',
+             _("override the current version used in the version check "
+               "and enable version check debugging mode")),
+            ('geometry', '',
+             _("specify window size and position, in the format: "
+               "WIDTHxHEIGHT+XOFFSET+YOFFSET")),
+            ])
+
+        if os.name == 'nt':
+            r.extend([
+                ('launch_on_startup', MYTRUE,
+                 _("Launch BitTorrent when Windows starts")),
+                ('minimize_to_tray', MYTRUE,
+                 _("Minimize to system tray")),            
             ])
 
     if ui in ('bittorrent-console', 'bittorrent-curses'):
@@ -216,6 +253,8 @@ def get_defaults(ui):
                "corresponding .torrent file")),
             ('parse_dir_interval', 60,
               _("how often to rescan the torrent directory, in seconds") ),
+            ('launch_delay', 0,
+             _("wait this many seconds after noticing a torrent before starting it, to avoid race with tracker")),
             ('saveas_style', 4,
               _("How to name torrent downloads: "
                 "1: use name OF torrent file (minus .torrent);  " 
